/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    AbstractLoader.java
 *    Copyright (C) 2002 Richard Kirkby
 *
 */

package weka.core.converters;

import weka.core.Instances;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Abstract class gives default implementation of setSource
 * methods. All other methods must be overridden.
 *
 * @author Richard Kirkby (rkirkby@cs.waikato.ac.nz)
 * @version $Revision: 1.6 $
 */
public abstract class AbstractSaver implements Saver
{

    /**
     * Default implementation throws an IOException.
     *
     * @param file the File
     * @throws IOException always
     */
    public void setDestination(File file) throws IOException
    {
        throw new IOException( "Setting File as source not supported" );
    }

    /**
     * Default implementation throws an IOException.
     *
     * @param input the input stream
     * @throws IOException always
     */
    public void setDestination(OutputStream input) throws IOException
    {
        throw new IOException( "Setting InputStream as source not supported" );
    }

    public void beginSave()
    {
        // Empty implementation
    }

    public void endSave() throws IOException
    {
        // Empty implementation
    }

    /**
     * Saves the structure (internally the header) of the data set.
     *
     * @param instances the set of Instances
     * @throws IOException if there is no source or saving fails
     */
    public abstract void saveStructure(Instances instances) throws IOException;

    /**
     * Save the full data set.
     *
     * @param instances the full data set as an Instances object
     * @throws IOException if there is an error during saving
     */
    public abstract void saveDataSet(Instances instances) throws Exception;
}
