package weka.core.converters;

import weka.core.Instances;

import java.io.IOException;
import java.io.File;
import java.io.OutputStream;

/**
 * Implementation of class
 * 
 * @author anilkpatro
 * @date Apr 1, 2004
 */
public interface Saver
{
    /**
     * Resets the Loader object and sets the source of the data set to be
     * the supplied File object.
     *
     * @param file the File
     * @exception IOException if an error occurs saving to a File.
     */
    void setDestination(File file) throws IOException;

    /**
     * Resets the Loader object and sets the source of the data set to be
     * the supplied InputStream.
     *
     * @param output the destination OutputStream
     * @exception IOException if this Saver cannot output to the stream.
     */
    void setDestination(OutputStream output) throws IOException;

    /**
     * Saves the structure (internally the header) of the data set.
     *
     * @param instances the set of Instances
     * @exception IOException if there is no source or saving fails
     */
    public void saveStructure(Instances instances) throws IOException;

    /**
     * Save the full data set.
     *
     * @param instances the full data set as an Instances object
     * @exception IOException if there is an error during saving
     */
    public void saveDataSet(Instances instances) throws Exception;

    public void beginSave();

    public void endSave() throws IOException;
}
