/*
 * CopyrightPlugin (c) 2004 Your Corporation. All Rights Reserved.
 */
package weka.gui.explorer;

import xmdv.dataproc.Dataset;
import xmdv.dataproc.AbstractDimension;
import xmdv.dataproc.DoubleDimension;
import xmdv.graphics.*;

import javax.swing.*;
import javax.swing.border.SoftBevelBorder;
import javax.swing.border.BevelBorder;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.*;

import gl4java.GLContext;
import weka.core.*;

/**
 * Implementation of class XMDVPanel
 * 
 * @author anilkpatro
 * @since Mar 14, 2004
 */
public class XMDVPanel extends JPanel implements FocusListener
{
    private final String flatParcoordString = "flat parcoord";
    private final String flatGlyphString = "flat glyph";
    private final String flatScatterString = "flat scatter";

    private final String flatDimstackString = "flat dimstack";

    private final String openString = "open";
    private final String closeString = "close";

    private Dataset dataset = null;

    public XMDVPanel()
    {
        if ( !GLContext.doLoadNativeLibraries( null, null, null ) ) {
            System.out.println( "could not load native libs" );
            return;
        }
        initComponents();

        canvas.setHoriScrollBar( scrollHorizontal );
        canvas.setVertScrollBar( scrollVertical );
        scrollHorizontal.addAdjustmentListener( canvas );
        scrollVertical.addAdjustmentListener( canvas );

        setDisplay( flatParcoordString );

        addFocusListener( this );
        setFocusable( true );
    }

    private void initComponents()
    {
        this.setLayout( new BorderLayout() );

        panelButtons = new javax.swing.JPanel();
        panelButtons.setLayout( new javax.swing.BoxLayout( panelButtons, javax.swing.BoxLayout.Y_AXIS ) );
        btngrpDisplayMode = new javax.swing.ButtonGroup();

        java.awt.event.ActionListener buttonListener = new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                buttonGroupActionPerformed( evt );
            }
        };

        btnFlatParcoord = new javax.swing.JRadioButton();
        btnFlatParcoord.setBackground( new java.awt.Color( 204, 204, 204 ) );
        btngrpDisplayMode.add( btnFlatParcoord );
        btnFlatParcoord.setBorder( new javax.swing.border.SoftBevelBorder( javax.swing.border.BevelBorder.RAISED ) );
        btnFlatParcoord.setBorderPainted( true );
        Image iconFlatParcoord = Toolkit.getDefaultToolkit()
                            .getImage(ClassLoader.getSystemResource("xmdv/images/imgpa.gif"));
        btnFlatParcoord.setIcon( new javax.swing.ImageIcon( iconFlatParcoord ) );
        btnFlatParcoord.setActionCommand( flatParcoordString );
        btnFlatParcoord.addActionListener( buttonListener );
        panelButtons.add( btnFlatParcoord );

        btnFlatGlyph = new javax.swing.JRadioButton();
        btngrpDisplayMode.add( btnFlatGlyph );
        btnFlatGlyph.setBorder( new javax.swing.border.SoftBevelBorder( javax.swing.border.BevelBorder.RAISED ) );
        btnFlatGlyph.setBorderPainted( true );
        Image iconFlatGlyph = Toolkit.getDefaultToolkit()
                            .getImage(ClassLoader.getSystemResource("xmdv/images/imgst.gif"));
        btnFlatGlyph.setIcon( new javax.swing.ImageIcon( iconFlatGlyph ) );
        btnFlatGlyph.setActionCommand( flatGlyphString );
        btnFlatGlyph.addActionListener( buttonListener );
        panelButtons.add( btnFlatGlyph );

        btnFlatScatter = new javax.swing.JRadioButton();
        btngrpDisplayMode.add( btnFlatScatter );
        btnFlatScatter.setBorder( new javax.swing.border.SoftBevelBorder( javax.swing.border.BevelBorder.RAISED ) );
        btnFlatScatter.setBorderPainted( true );
        Image iconFlatScatter = Toolkit.getDefaultToolkit()
                            .getImage(ClassLoader.getSystemResource("xmdv/images/imgsc.gif"));
        btnFlatScatter.setIcon( new javax.swing.ImageIcon( iconFlatScatter ) );
        btnFlatScatter.setActionCommand( flatScatterString );
        btnFlatScatter.addActionListener( buttonListener );
        panelButtons.add( btnFlatScatter );

        btnFlatDimstack = new javax.swing.JRadioButton();
        btngrpDisplayMode.add( btnFlatDimstack );
        btnFlatDimstack.setBorder( new javax.swing.border.SoftBevelBorder( javax.swing.border.BevelBorder.RAISED ) );
        btnFlatDimstack.setBorderPainted( true );
        Image iconFlatDimstack = Toolkit.getDefaultToolkit()
                            .getImage(ClassLoader.getSystemResource("xmdv/images/imgdi.gif"));
        btnFlatDimstack.setIcon( new javax.swing.ImageIcon( iconFlatDimstack ) );
        btnFlatDimstack.setActionCommand( flatDimstackString );
        btnFlatDimstack.addActionListener( buttonListener );
        panelButtons.add( btnFlatDimstack );

        this.add( panelButtons, java.awt.BorderLayout.EAST );

        panelCanvas = new javax.swing.JPanel();
        panelCanvas.setLayout( new java.awt.BorderLayout() );

        scrollVertical = new javax.swing.JScrollBar();
        scrollVertical.setToolTipText( "null" );
        panelCanvas.add( scrollVertical, java.awt.BorderLayout.EAST );

        scrollHorizontal = new javax.swing.JScrollBar();
        scrollHorizontal.setOrientation( javax.swing.JScrollBar.HORIZONTAL );
        panelCanvas.add( scrollHorizontal, java.awt.BorderLayout.SOUTH );

        canvas = new grCanvas();
        panelCanvas.add( canvas, java.awt.BorderLayout.CENTER );

        this.add( panelCanvas, java.awt.BorderLayout.CENTER );
    }

    private void setDisplay(String str)
    {
        btnFlatParcoord.setBorder( new SoftBevelBorder( str.equals( flatParcoordString ) ? BevelBorder.LOWERED : BevelBorder.RAISED ) );
        btnFlatGlyph.setBorder( new SoftBevelBorder( str.equals( flatGlyphString ) ? BevelBorder.LOWERED : BevelBorder.RAISED ) );
        btnFlatScatter.setBorder( new SoftBevelBorder( str.equals( flatScatterString ) ? BevelBorder.LOWERED : BevelBorder.RAISED ) );
        btnFlatDimstack.setBorder( new SoftBevelBorder( str.equals( flatDimstackString ) ? BevelBorder.LOWERED : BevelBorder.RAISED ) );

        FlatDisplay dis = null;
        if ( str.equals( flatParcoordString ) ) {
            dis = new FlatParcoord( canvas, new ColorManager(), dataset );
        } else if ( str.equals( flatGlyphString ) ) {
            dis = new FlatGlyph( canvas, new ColorManager(), dataset );
        } else if ( str.equals( flatScatterString ) ) {
            dis = new FlatScatter( canvas, new ColorManager(), dataset );
        } else if ( str.equals( flatDimstackString ) ) {
            dis = new FlatDimstack( canvas, new ColorManager(), dataset );
        }
        canvas.setDisplay( dis );
        canvas.refresh();
    }

    private void buttonGroupActionPerformed(java.awt.event.ActionEvent evt)
    {
        String str = evt.getActionCommand();
        setDisplay( str );
    }

    // Variables declaration
    private javax.swing.JPanel panelButtons;
    private javax.swing.ButtonGroup btngrpDisplayMode;
    private javax.swing.JRadioButton btnFlatParcoord;
    private javax.swing.JRadioButton btnFlatGlyph;
    private javax.swing.JRadioButton btnFlatScatter;
    private javax.swing.JRadioButton btnFlatDimstack;

    private javax.swing.JPanel panelCanvas;
    private javax.swing.JScrollBar scrollVertical;
    private javax.swing.JScrollBar scrollHorizontal;
    private grCanvas canvas;

    public void setInstances(Instances instances)
    {
        dataset = new WekaDataset( instances );
        setDisplay( flatParcoordString );
    }

    public void focusGained(FocusEvent e)
    {
        canvas.requestFocus();
    }

    public void focusLost(FocusEvent e)
    {
    }
}

class WekaDataset extends Dataset
{
    WekaDataset(Instances instances)
    {
        super();

        allDimNum = instances.numAttributes();
        dataNum = instances.numInstances();

        //new dimensions
        allDims = new AbstractDimension[allDimNum];

        //read name of dimensions
        for ( int i = 0; i < allDimNum; i++ ) {
            allDims[i] = new DoubleDimension();
            allDims[i].setName( instances.attribute( i ).name() );
        }

        //read dimension minimum, maximum, and dimension cardinality for dimension stacking
        for ( int i = 0; i < allDimNum; i++ ) {
            AttributeStats as = instances.attributeStats( i );

            if (as.numericStats != null) {
                allDims[i].setMin( Utils.doubleToString( as.numericStats.min, 3 ) );
                allDims[i].setMax( Utils.doubleToString( as.numericStats.max, 3 ) );
            } else {
                allDims[i].setMin( Utils.doubleToString( 0, 3 ) );
                allDims[i].setMax( Utils.doubleToString( as.distinctCount-1, 3 ) );
            }
            allDims[i].setCardinality( 1 );
            allDims[i].addRim();
        }

        int i, j;

        norDataBuf = new double[dataNum][allDimNum];
        for ( i = 0; i < dataNum; i++ ) {
            Instance inst = instances.instance( i );
            for ( j = 0; j < allDimNum; j++ ) {
                norDataBuf[i][j] = allDims[j].getNorValue( Utils.doubleToString( inst.value( j ), 3 ) );
            }
        }

        initAllBrush();
        initBrushOperAndMarks();
        initCurrDimIndexList();
    }
}