package xmdv.graphics;



/***********************************************************

**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003

**  All Rights Reserved.

**

**  Author: Jing Yang

**  Date:   06/2003

***********************************************************/



import xmdv.dataproc.AbstractDimension;
import xmdv.dataproc.Brush;
import xmdv.dataproc.BrushOper;



/**

 * Abstract class for displaying and interactively exploring data items 

 * in their flat form (opposite to their clustered (hierarchical) form. 

 *

 * @author  Jing Yang

 * @version v1.0

 */

public abstract class FlatDisplay extends GeneralDisplay {

    // Variables declaration

    

    /** number of dimensions to be drawn */

    protected int dimNum;

    

    /** dimensions to be drawn */

    protected AbstractDimension[] dims;

    

    /** number of data items to be drawn */

    protected int dataNum;

    

    /** brush to be drawn */

    protected Brush brush;

    

    /** mouse status: idling */

    protected static final int NONE = 0;

    /** mouse status: canvas panning */

    protected static final int CANVAS_PAN = 1;

    /** mouse status: resizing brush top */

    protected static final int RESIZE_BRUSH_TOP = 2;

    /** mouse status: resizing brush bottom */

    protected static final int RESIZE_BRUSH_BOTTOM = 3;

    /** mouse status: resizing brush */

    protected static final int RESIZE_BRUSH = 4;

    /** mouse status: translating brush */

    protected static final int TRANSLATE_BRUSH = 5;

    /** mouse status: multi-selection */

    protected static final int MULTI_SELECT = 6;

    

    /** current mouse status */

    protected int inputMode;

    /** true if need to undo last drawing*/

    protected boolean xor_needs_undo = false;

    /** case of quick draw */

    protected int quickDrawCase;

    /** data to be quick drawn */

    protected double[] quickDrawNorData;

    /** index of the data to be quick drawn */

    protected int quickDrawDataIndex;

    /** color of quick drawn */

    protected java.awt.Color quickDrawColor;

    // End of variables declaration

    

     

    /** Creates a new instance of FlatDisplay */

    public FlatDisplay() {

    }

        

    /** Sets parameters of the display */

    protected void setParameters() {

        if (dataset != null) {

            dataNum = dataset.getDataNum();

            dimNum = dataset.getCurrDimNum();

            dims = dataset.getCurrDims();

            brush = dataset.getCurrBrush();

        } else {

            dataNum = 0;

            dimNum = 0; 

            dims = null;

            brush = null;

        }

    }

    

    /** Draws the display on the canvas */

    protected void draw() {

        prepareCanvas();

    

    	drawBrush();

    

        drawRegularData();

     

        drawHighlightedData();

    

    	drawAxes();

    }

    

    /** Clears canvas using background color */

    protected void prepareCanvas() {

        canvas.clear(colorManager.getColor("background"));

    }

    

    /** Draws brush */

    protected void drawBrush() {

        drawSingleBrush(brush);

    }

    

    /** Draws regular data */

    protected void drawRegularData() {

        double[] norData;

        

        canvas.setForegroundColor(colorManager.getColor("data"));

        for (int i = 0; i < dataNum; i++) {

            if (dataset.getMark(i) == BrushOper.REGULAR) {

                norData = dataset.getCurrNorData(i);

                drawSingleData(norData, i);

            }

        }

        drawRegularDataCleanUp();

    }

    

    /** Cleans up after drawing regular data */

    protected void drawRegularDataCleanUp() {

    }

    

    /** Draws highlighted data */

    protected void drawHighlightedData()

    {

        double[] norData;

        

        canvas.setForegroundColor(colorManager.getColor("highlight"));

        for (int i=0; i < dataNum; i++) {

            if (dataset.getMark(i) == BrushOper.HIGHLIGHTED) {

                norData = dataset.getCurrNorData(i);

                drawSingleData(norData, i);

            }

        }

        drawHighlightedDataCleanUp();

    }

    

    /** Cleans up after drawing highlighted data */

    protected void drawHighlightedDataCleanUp() {

    }

    

    /** Handles key press event */

    protected void inputKeyPress(int key) {

        if (canvas.shiftkey) {

            // Go into multiselect mode

            if (!dataset.isMultiSelectOn()) {

                dataset.multiSelectSetState(true);

                canvas.setQuickDraw(true);

                canvas.setHandCursor();

            }

            return;

        }

    }

    

    /** Handles key release event */

    protected void inputKeyRelease(int key) {

        if (key == java.awt.event.KeyEvent.VK_SHIFT) {

            // Exit multiselect mode

            dataset.multiSelectSetState(false);

            canvas.setQuickDraw(false);

            dataset.refreshAllCanvas();

            canvas.setDefaultCursor();

            return;

        }

    }

    

    /** Prepares quick draw case 0 */

    public void setQuickDrawCase0(double[] norData, int dataIndex, java.awt.Color color) {

        quickDrawCase = 0;

        quickDrawNorData = norData;

        quickDrawDataIndex = dataIndex;

        quickDrawColor = color;

    }

    

    /** Quick draws */

    public void quickDraw() {

        if (quickDrawCase == 0) {

            canvas.setForegroundColor(quickDrawColor);

            drawSingleData(quickDrawNorData, quickDrawDataIndex);

        } else if (quickDrawCase == 1) {

        }

        

    }

    

    /** Draws axes, or grid, and legends */

    protected abstract void drawAxes();

    /**

     * Draws a single data

     * @param norData data value

     * @param dataIndex index of the data 

     */

    protected abstract void drawSingleData(double[] norData, int dataIndex);

    /** Draws a single brush */

    protected abstract void drawSingleBrush(Brush brush);

}

