package xmdv.graphics;



/***********************************************************

**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003

**  All Rights Reserved.

**

**  Author: Jing Yang

**  Date:   06/2003

***********************************************************/



import java.awt.geom.Point2D;

import xmdv.dataproc.Brush;
import xmdv.dataproc.Brush1D;
import xmdv.dataproc.Dataset;



/**

 * This class is an implementation of the abstract class FlatDisplay using 

 * the Pararell Coordinates technique.

 *

 * @author  Jing Yang

 * @version v1.0

 */

public class FlatParcoord extends FlatDisplay{

    private Point2D.Double prevPosition;

    private int prevDim;

    private double prevNorValue;

    private Point2D.Double[] pts;

    private double[] axesPos;

    private double[] axesInterval;

    

    private double multiselRadius = 0.01;

    

    /* inits: pt = point, pts = points, lo = low, hi = high, num = number 

     *        nor = normalized, prev = previous, dimIndex = dimension 

     *        min = minimum, max = maximum */

        

    /** Creates a new instance of FlatParcoord */

    public FlatParcoord () {

    }

    

    public FlatParcoord(grCanvas canvas, ColorManager colorManager, Dataset dataset) {

        setCanvas(canvas);

        setColorManager(colorManager);

        setDataset(dataset);

    }

    

    protected boolean isDimensionalityValid()  

    {

        if ( dataset.getCurrDimNum() < 2 ) {

            System.out.println("No sufficent dimensions for this kind of display.");

            return false;

        } else 

            return true;

    }

    

    protected void setParameters() {

        super.setParameters();

        

        if (dimNum != 0) {

            axesPos = new double[dimNum];

            axesInterval = new double[dimNum];

            double interval = 1./dimNum;

            for (int i = 0; i < dimNum; i++) {

                axesInterval[i] = interval;

                axesPos[i] = interval * i + 0.5 * interval;

            }

            axesInterval[dimNum-1] = 0.5 * interval + grCanvas.BORDER;

            

            pts = new Point2D.Double[dimNum];

        }    

    }

    

    

    protected void drawSingleData(double[] norData, int dataIndex) {

        for ( int i = 0; i < dimNum; i++ )

        {

            pts[i] = getPosition(i, norData[i]);

            

        }

        canvas.drawLines(pts);

    }

    

    

    protected void drawSingleBrush(Brush brush) {

        canvas.setForegroundColor(colorManager.getColor("brush"));

    

        Point2D.Double[] hiPts = new Point2D.Double[dimNum];

        Point2D.Double[] loPts = new Point2D.Double[dimNum];

        for (int j = 0; j < dimNum; j++)

        {

            hiPts[j] = getPosition(j, brush.getNorMax(j));

            loPts[j] = getPosition(j, brush.getNorMin(j));

        }

    

        canvas.drawQuadStrip(hiPts, loPts);

    }

    

    protected void drawAxes()

    {

        canvas.setLineWidth(1.);

    

        // Draw the vertical axes

        int i;

        for (i = 0; i < dimNum; i++) {

            canvas.setForegroundColor(colorManager.getColor("grid0"));

            Point2D.Double pt1, pt2;

            pt1 = getPosition(i, 0);

            pt2 = getPosition(i, 1);

            canvas.drawLine(pt1, pt2);

        }

    

        //Draw axes names and mins and maxes

        drawParcoordAxesNameMinMax();

    }

    

    protected void inputMouseMotion(Point2D.Double pt)

    {

        if (inputMode == CANVAS_PAN) {

            canvas.pan(prevPosition, pt);

            canvas.refresh();

        }

        else {

            int dimIndex = getDim(pt);

            if ( dimIndex < 0 )

                return;

            double norValue = getNorValue(pt);

    

            parcoordCaption(dimIndex, norValue);

        

            switch (inputMode) {

                case RESIZE_BRUSH_TOP:

                case RESIZE_BRUSH_BOTTOM:

                    if (xor_needs_undo) 

                        inputParcoordResizeBrush(prevDim, prevNorValue);

                    inputParcoordResizeBrush(dimIndex, norValue);

                    dataset.refreshAllCanvas();

                    break;

        

                case TRANSLATE_BRUSH:

                    if (xor_needs_undo) 

                        inputParcoordMoveBrush(prevDim, prevNorValue);

                    inputParcoordMoveBrush(dimIndex, norValue);

                    dataset.refreshAllCanvas();

                    break;

        

                case MULTI_SELECT:

                    inputParcoordMultiSelect(dimIndex, norValue);

                    dataset.refreshAllCanvas();

                    return;

        

                default: break;

            }

            prevPosition = pt;

            prevDim = dimIndex;

            prevNorValue = norValue;

        }    

    }

    

    

    protected void inputButtonPress(int button, Point2D.Double pt) {

        inputMode = NONE;

        prevPosition = pt;

    

        if ((!canvas.altkey) && (!canvas.controlkey) && button == 3) {

            inputMode = CANVAS_PAN;

            canvas.setMoveCursor();

            return;

        }  

        int dimIndex = getDim(pt);

        if ( dimIndex < 0 )

            return;

        double norValue = getNorValue(pt);

    

        if (canvas.shiftkey && button == 1) {

            inputMode = MULTI_SELECT;

            inputParcoordMultiSelect(dimIndex,norValue);

            return;

        } 

    

        if (button == 1) {

            double norMin = brush.getNorMin(dimIndex);

            double norMax = brush.getNorMax(dimIndex);

            if (Math.abs(norMax - norValue) < Math.abs(norValue - norMin) ) {

                inputMode = RESIZE_BRUSH_TOP;

            } else {

                inputMode = RESIZE_BRUSH_BOTTOM;

            }

        } else if (button == 2) {

            inputMode = TRANSLATE_BRUSH;

        }

    

        inputMouseMotion(pt);

    }

    

    

    protected void inputButtonRelease(int button, Point2D.Double pt)

    {

        switch (inputMode) {

    

            case CANVAS_PAN:

                canvas.setDefaultCursor();

                break;

    

            case MULTI_SELECT:

                break;

    

            case RESIZE_BRUSH_TOP:

            case RESIZE_BRUSH_BOTTOM:

            case TRANSLATE_BRUSH:

                dataset.brush();

                canvas.setDefaultCursor();

                dataset.refreshAllCanvas();

                break;

    

            default: break;

        }

    

        inputMode = NONE;

        xor_needs_undo = false;

    }

    

    // get the position of a normalized value of a dimension

    protected Point2D.Double getPosition(int i, double value)

    {

        return new Point2D.Double(axesPos[i], value);

    }

    

    protected int getDim(Point2D.Double pt)

    {

        int dimIndex = -1;

        double x = pt.getX();

        double y = pt.getY();

        

        if ( y > 0. && y < 1.0 && x > 0. && x < 1.0 ) {

            dimIndex = dimNum-1;

            for ( int i = 0; i < dimNum-1; i++ ) {

                if ( x < axesPos[i] + axesInterval[i]/2 )

                {

                    dimIndex = i;

                    break;

                }

            }

        }

        return dimIndex;

    }

    

    protected double getNorValue(Point2D.Double pt)

    {

        return pt.getY();

    }

    

    

    // Draw parallel coordinate axes name and min max.

    protected void drawParcoordAxesNameMinMax()

    {

        canvas.setForegroundColor(colorManager.getColor("text"));

    

        double x, y;

        Point2D.Double pt;

        double fontHeight = canvas.getFontHeight('H');

        double pad = fontHeight * 0.1;

        for (int i = 0; i < dimNum; i++)

        {

            pt = getPosition(i, 1);

            x = pt.getX() + pad;

            

            String dimName = dims[i].getName();

            y = 1 + grCanvas.BORDER * 0.6;

            canvas.drawHoriString(new Point2D.Double(x, y),

                                  axesInterval[i] - pad, dimName);

    

            String origMaxString = dims[i].getOrigMaxString();

            double origMaxValue = dims[i].getNorValue(origMaxString);

            pt = getPosition(i, origMaxValue);

            canvas.setPointSize(2.0);

            canvas.drawPoint(pt);

            y = pt.getY() + 1.1 * fontHeight;

            canvas.drawHoriString(new Point2D.Double(x, y), axesInterval[i] - pad, 

                                  dims[i].getValueMessage(origMaxValue));

            String origMinString = dims[i].getOrigMinString();

            double origMinValue = dims[i].getNorValue(origMinString);

            pt = getPosition(i, origMinValue);

            canvas.setPointSize(2.0);

            canvas.drawPoint(pt);

            y = pt.getY() - 0.1 * fontHeight;

            canvas.drawHoriString(new Point2D.Double(x, y), axesInterval[i] - pad, 

                                  dims[i].getValueMessage(origMinValue));

        }

    }

    

    

    protected void inputParcoordResizeBrush(int dimIndex, double norValue)

    {

        canvas.setQuickDraw(true);

    

        switch (inputMode) {

        case RESIZE_BRUSH_TOP: {

            brush.setNorMax(dimIndex, norValue);

    

            // Resize top

            Point2D.Double pt = getPosition(dimIndex, brush.getNorMax(dimIndex));

            if (dimIndex > 0) {

                Point2D.Double pt1 = getPosition(dimIndex - 1, brush.getNorMax(dimIndex - 1));

                canvas.drawLine(pt1, pt);

            }

    

            if (dimIndex < dimNum-1) {

                Point2D.Double pt2 = getPosition(dimIndex + 1, brush.getNorMax(dimIndex + 1));

                canvas.drawLine(pt2, pt);

            }

    

            } break;

    

        case RESIZE_BRUSH_BOTTOM: {

            brush.setNorMin(dimIndex, norValue);

    

            // Resize bottom

            Point2D.Double pt = getPosition(dimIndex, brush.getNorMin(dimIndex));

            if (dimIndex > 0) {

                Point2D.Double pt1 = getPosition(dimIndex - 1, brush.getNorMin(dimIndex - 1));

                canvas.drawLine(pt1, pt);

            }

    

            if (dimIndex < dimNum - 1) {

                Point2D.Double pt2 = getPosition(dimIndex + 1, brush.getNorMin(dimIndex + 1));

                canvas.drawLine(pt2, pt);

            }

    

            } break;

    

        default:

            break;

        }

    

        canvas.setQuickDraw(false);

        xor_needs_undo = true;

    }

    

    

    protected void inputParcoordMoveBrush(int dimIndex, double norValue)

    {

        Brush1D tempBrush1D = brush.getBrush1D(dimIndex).copy();

        tempBrush1D.shift(norValue - brush.getNorCenter(dimIndex));

    

        canvas.setQuickDraw(true);

        

        Point2D.Double loPt = getPosition(dimIndex, tempBrush1D.getMin());

        Point2D.Double hiPt = getPosition(dimIndex, tempBrush1D.getMax()); 

    

        if (dimIndex > 0) {

            Point2D.Double loPt1 = getPosition(dimIndex - 1, brush.getNorMin(dimIndex - 1));

            Point2D.Double hiPt1 = getPosition(dimIndex - 1, brush.getNorMax(dimIndex - 1));

            canvas.drawLine(loPt1, loPt);

            canvas.drawLine(hiPt1, hiPt);

        }

    

        if (dimIndex < dimNum - 1) {

            Point2D.Double loPt2 = getPosition(dimIndex + 1, brush.getNorMin(dimIndex + 1));

            Point2D.Double hiPt2 = getPosition(dimIndex + 1, brush.getNorMax(dimIndex + 1));

            canvas.drawLine(loPt2, loPt);

            canvas.drawLine(hiPt2, hiPt);

        }

    

        canvas.setQuickDraw(false);

    

        brush.shift(dimIndex, norValue - brush.getNorCenter(dimIndex));

        xor_needs_undo = true;

    }

    

    protected void inputParcoordMultiSelect(int dimIndex, double norValue)

    {

        for (int i = 0; i < dataNum; i++)

        {

            double[] norData = dataset.getCurrNorData(i);

    

            if (Math.abs(norData[dimIndex] - norValue) < multiselRadius)

            {

                dataset.multiSelectAddPoint(i);

                setQuickDrawCase0(norData, i, colorManager.getColor("paint"));

                canvas.paintImmediately(0, 0, canvas.getWidth(), canvas.getHeight()); 

            }

        }

    }

    

    

    protected void parcoordCaption(int dimIndex, double norValue) {

        String valueMessage = dims[dimIndex].getNamedValueMessage(norValue);

        canvas.showMessage(valueMessage);

    }

}

